/*
TimeSystem.java
Class responsible for round time measuring in the
Kleptomania game

Green Team (the good one) October 18, 2004
 */




public class TimeSystem extends Thread
{


    int round  = 0;
    int period = 0;
    int timeToWait = 0;
    boolean pause = false;

    TimePublisher publisher = null;


    final String vars = "";

    // time per period in tenths of seconds
    int WAIT_LEN   = 50;
    int PERIOD_LEN = 40;

    // names of the periods
    final int BLUE    = 1;
    final int RED     = 0;
    final int GREEN   = 2;
    final int YELLOW  = 3;

    // number of periods per round, the
    // last period is pause, and has to be
    // triggered
    final int PERIODS = 4;

    public TimeSystem()
    {
	round = 0;
	period = 0;
	timeToWait = WAIT_LEN;
	System.out.println("TimeSystem initiated");
    }


    public void run()
    {
      /* event loop */
      while( true )
      {
        try{
          handleEvents();
        }
        catch(InterruptedException e){}
      }
    }


    synchronized public void handleEvents( ) throws InterruptedException
    {
      // lock on pause, and do not do active waiting
      if( pause )
      {
        System.out.println("Paused");
        wait();
      }
      // end of period
      if( !pause && timeToWait == 0 )
      {
          period = (period + 1) % PERIODS;
          //System.out.println("PERIOD: " + period);
          timeToWait = PERIOD_LEN;
          if (period == 0)
          {
	      round += 1;
	      setPause(true);
          }
      }
      // active waiting on time loops, as we do not use any global glock to
      // measure time
      // NOTE: if not pause then the condition is always true
      if( !pause && timeToWait > 0 )
      {
	if( publisher != null ) publisher.publishTick( pause, round, period, timeToWait );
        wait(100);
        timeToWait-=1;
      }

    }


    synchronized public void setPause( boolean pval )
    {
	      pause = pval;
	      if( pause && publisher != null ) publisher.publishTick( pause, round, period, timeToWait );
	      if( pval == false ) notify();
    }


    public void setPublisher( TimePublisher pbls )
    {
      publisher = pbls;
    }

    public void reset()
    {
      round = 0;
      period = 0;
      timeToWait = WAIT_LEN;
      setPause( true );
    }


    public void setPeriodLen( int l ) throws NumberFormatException
    {
	if( l <= 0 ) throw new NumberFormatException("Period length must be positive");
	PERIOD_LEN = l*10;
	
    }

}
